/*
 * Copyright (C) 2007 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ifndef bcmst_h
#define bcmst_h
#ident "$Id: bcmst.h,v 1.11 2007/05/28 05:24:45 nalin Exp $"

#include <nspr.h>
#include <nss.h>

#include "aabag.h"
#include "pkinit.h"
#include "commont.h"

struct attribute {
	SECItem attr_type;
	SECItem **attr_values;
};

enum signer_type {
	signer_type_issuer_and_serial_number = 1,
	signer_type_subject_key_identifier = 2,
};

struct signer_identifier {
	int signer_type; /* actually an "enum signer_type" */
	struct {
		struct issuer_and_serial_number issuer_and_serial_number;
		SECItem subject_key_identifier;
	} signer_value;
};

struct signer_info {
	SECItem version;
	struct signer_identifier sid;
	struct algorithm_identifier digest_algorithm;
	SECItem signed_attrs;
	struct algorithm_identifier signature_algorithm;
	SECItem signature;
	SECItem unsigned_attrs;
};

struct content_info {
	SECItem content_type;
	SECItem content;
};

struct signed_data {
	SECItem version;
	struct algorithm_identifier **digest_algorithms;
	struct content_info encap_content_info;
	SECItem **certificates;
	SECItem **crls;
	struct signer_info **signer_infos;
};

struct external_principal_identifier {
	SECItem der_subject_name;
	SECItem issuer_and_serial_number;
	SECItem subject_key_identifier;
};

enum recipient_identifier_type {
	recipient_identifier_type_issuer_and_serial_number = 1,
	recipient_identifier_type_subject_key_identifier = 2,
};

struct recipient_identifier {
	int recipient_identifier_type; /* actually an "enum recipient_identifier_type" */
	struct {
		struct issuer_and_serial_number issuer_and_serial_number;
		SECItem subject_key_identifier;
	} recipient_identifier_value;
};

struct key_trans_recipient_info {
	SECItem version;

	struct recipient_identifier rid;
	struct algorithm_identifier key_encryption_algorithm;
	SECItem encrypted_key;
};

enum recipient_info_type {
	recipient_info_type_key_trans = 1,
	recipient_info_type_key_agree = 2,
	recipient_info_type_kek_recipient = 3,
};

struct recipient_info {
	int recipient_info_type; /* actually an "enum recipient_info_type" */
	struct {
		struct key_trans_recipient_info ktri;
		SECItem der_kari;
		SECItem der_kekri;
	} recipient_info_value;
};

struct encrypted_content_info {
	SECItem content_type;
	struct algorithm_identifier content_encryption_algorithm;
	SECItem encrypted_content;
};

struct enveloped_data {
	SECItem version;
	SECItem der_originator_info;
	struct recipient_info **recipient_infos;
	struct encrypted_content_info encrypted_content_info;
	SECItem der_unsigned_attrs;
};

struct encrypted_data {
	SECItem version;
	struct encrypted_content_info encrypted_content_info;
	SECItem der_unsigned_attrs;
};

/* ASN.1 templates */
extern const SEC_ASN1Template bcms_content_info_template[];
extern const SEC_ASN1Template bcms_primitive_content_info_template[];
extern const SEC_ASN1Template bcms_encrypted_content_info_template[];
extern const SEC_ASN1Template bcms_enveloped_data_template[];
extern const SEC_ASN1Template bcms_encrypted_data_template[];
extern const SEC_ASN1Template bcms_external_principal_identifiers_template[];
extern const SEC_ASN1Template bcms_key_trans_recipient_info_template[];
extern const SEC_ASN1Template bcms_recipient_identifier_template[];
extern const SEC_ASN1Template bcms_recipient_infos_template[];
extern const SEC_ASN1Template bcms_sequence_of_external_principal_identifier_template[];
extern const SEC_ASN1Template bcms_signed_data_template[];
extern const SEC_ASN1Template bcms_signer_identifier_template[];
extern const SEC_ASN1Template bcms_signer_infos_template[];

/* Encoder/decoder functions. */
struct content_info *bcms_build_content_info(PLArenaPool *pool,
					     SECOidTag content_type_tag,
					     SECItem *content_data);
struct content_info *bcms_build_content_info_oid(PLArenaPool *pool,
						 SECItem *content_type_oid,
						 SECItem *content_data);
SECItem *bcms_encode_set_of_attributes(struct module_context *mcontext,
				       PLArenaPool *pool,
				       struct attribute **attr);
SECItem *bcms_encode_signed_attributes(struct module_context *mcontext,
				       PLArenaPool *pool,
				       struct attribute **attr);
SECItem *bcms_encode_content_info(struct module_context *mcontext,
				  PLArenaPool *pool, struct content_info *ci);
SECItem *bcms_encode_primitive_content_info(struct module_context *mcontext,
					    PLArenaPool *pool,
					    struct content_info *ci);
SECItem *bcms_encode_signed_data(struct module_context *mcontext,
				 PLArenaPool *pool, struct signed_data *sd);
SECItem *bcms_encode_enveloped_data(struct module_context *mcontext,
				    PLArenaPool *pool,
				    struct enveloped_data *ed);
SECItem *bcms_encode_encrypted_data(struct module_context *mcontext,
				    PLArenaPool *pool,
				    struct encrypted_data *ed);
struct attribute **bcms_decode_set_of_attributes(struct module_context *mcontext,
						 PLArenaPool *pool,
						 SECItem *item);
struct attribute **bcms_decode_signed_attributes(struct module_context *mcontext,
						 PLArenaPool *pool,
						 SECItem *item);
struct content_info *bcms_decode_content_info(struct module_context *mcontext,
					      PLArenaPool *pool, SECItem *item);
struct signed_data *bcms_decode_signed_data(struct module_context *mcontext,
					    PLArenaPool *pool, SECItem *item);
struct external_principal_identifier **bcms_decode_sequence_of_external_principal_identifier(struct module_context *mcontext,
											     PLArenaPool *pool, SECItem *item);
struct enveloped_data *bcms_decode_enveloped_data(struct module_context *mcontext,
						  PLArenaPool *pool,
						  SECItem *item);
struct encrypted_data *bcms_decode_encrypted_data(struct module_context *mcontext,
						  PLArenaPool *pool,
						  SECItem *item);

/* Create a signed data structure. */
SECItem *bcms_create_signed_data(struct module_context *mcontext,
				 PLArenaPool *pool,
				 const SECItem *econtent_type,
				 SECItem *data,
				 CERTCertDBHandle *certdb,
				 struct aa_bag *bag,
				 CERTCertificate *signer,
				 int version,
				 SECOidTag digest_algorithm,
				 SECOidTag signature_algorithm,
				 PK11SlotInfo *preferred_slot,
				 PK11PasswordFunc pwc,
				 struct pkinit_pwcb_args *pwcb_args);

/* Verify a signed data structure. */
SECStatus bcms_verify_signed_data(struct module_context *mcontext,
				  PLArenaPool *pool,
				  CERTCertDBHandle *certdb,
				  struct signed_data *signed_data,
				  CERTCertificate **lone_signer,
				  time_t *lone_signing_time);

/* Create an enveloped data structure. */
SECItem *bcms_create_enveloped_data(struct module_context *mcontext,
				    PLArenaPool *pool,
				    CERTCertDBHandle *certdb,
				    krb5_context kcontext,
				    SECItem *content_type,
				    SECOidTag preferred_cek_algorithm,
				    SECItem *data,
				    SECItem *recipient_certificate);

/* Extract the data from an enveloped data structure. */
SECItem *bcms_extract_enveloped_data(struct module_context *mcontext,
				     PLArenaPool *pool,
				     CERTCertDBHandle *certdb,
				     struct enveloped_data *env,
				     PK11SlotInfo *preferred_slot,
				     struct aa_bag *bag,
				     PK11PasswordFunc pwcb,
				     struct pkinit_pwcb_args *pwcb_args);

struct external_principal_identifier **
bcms_make_external_principal_identifier_list(struct module_context *mcontext,
					     PLArenaPool *pool,
					     struct external_principal_identifier **list,
					     CERTCertificate *cert);
struct external_principal_identifier **bcms_get_trusted_client_certifier_list(struct module_context *mcontext,
									      PLArenaPool *pool,
									      struct pkinit_pwcb_args *pwcb_args);
struct external_principal_identifier **bcms_get_trusted_kdc_certifier_list(struct module_context *mcontext,
									   PLArenaPool *pool,
									   struct pkinit_pwcb_args *pwcb_args);

#endif
