/*
 * Copyright (C) 2006 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ifndef pkinitt_h
#define pkinitt_h
#ident "$Id: pkinitt.h,v 1.20 2007/05/28 05:26:24 nalin Exp $"

#include <nspr.h>

#include <nss.h>
#include <cert.h>
#include <certdb.h>
#include <certt.h>
#include <cms.h>
#include <cryptohi.h>
#include <keyhi.h>
#include <pk11pub.h>
#include <secasn1.h>
#include <sechash.h>

#include <krb5/krb5.h>
#include "aabag.h"
#include "commont.h"
#include "bcmst.h"

struct realm {
	SECItem name;
};

struct principal_name {
	SECItem name_type;
	SECItem **name_string;
};

struct draft_pkauthenticator {
	struct principal_name kdc_name;
	struct realm kdc_realm;
	SECItem cusec;
	SECItem ctime;
	SECItem nonce;
};

struct draft_auth_pack {
	struct draft_pkauthenticator pkauthenticator;
	SECItem client_public_value; /* a subject_public_key_info */
};

struct pkauthenticator {
	SECItem cusec;
	SECItem ctime;
	SECItem nonce;
	SECItem pa_checksum;
};

struct auth_pack {
	struct pkauthenticator pkauthenticator;
	SECItem client_public_value; /* a subject_public_key_info */
	struct algorithm_identifier **supported_cms_types;
	SECItem client_dh_nonce;
};

struct kerberos_principal_name {
	struct realm realm;
	struct principal_name principal_name;
};

enum trusted_ca_type {
	trusted_ca_type_principal_name = 1,
	trusted_ca_type_name = 2,
	trusted_ca_type_issuer_and_serial_number = 3,
};

struct trusted_cas {
	int trusted_ca_type; /* actually an "enum trusted_ca_type" */
	struct {
		struct kerberos_principal_name principal_name;
		SECItem name;
		struct issuer_and_serial_number issuer_and_serial;
	} trusted_ca_value;
};

struct draft_pa_pk_as_req {
	struct content_info signed_auth_pack;
	struct trusted_cas **trusted_cas;
	struct issuer_and_serial_number *kdc_cert, *encryption_cert;
};

struct pa_pk_as_req {
	SECItem signed_auth_pack;
	struct external_principal_identifier **trusted_certifiers;
	SECItem *kdc_pk_id;
};

enum draft_pa_pk_as_rep_type {
	draft_pa_pk_as_rep_type_enc_key_pack = 1,
	draft_pa_pk_as_rep_type_dh_signed_data = 2,
};

struct draft_pa_pk_as_rep {
	int draft_pa_pk_as_rep_type; /* actually an "enum pa_pk_as_rep_type" */
	struct {
		struct content_info enc_key_pack;
		struct content_info dh_signed_data;
	} draft_pa_pk_as_rep_value;
};

struct draft_kdc_dh_key_info {
	SECItem nonce;
	SECItem subject_public_key;
};

struct kdc_dh_key_info {
	SECItem subject_public_key;
	SECItem nonce;
	SECItem dh_key_expiration;
};

enum pa_pk_as_rep_type {
	pa_pk_as_rep_type_dh_info = 3,
	pa_pk_as_rep_type_enc_key_pack = 4,
};

struct dh_rep_info {
	SECItem dh_signed_data;
	SECItem *server_dh_nonce;
};

struct pa_pk_as_rep {
	int pa_pk_as_rep_type; /* actually an "enum pa_pk_as_rep_type" */
	struct {
		struct dh_rep_info dh_info;
		SECItem enc_key_pack;
	} pa_pk_as_rep_value;
};

struct encryption_key {
	SECItem keytype;
	SECItem keyvalue;
};

struct draft_reply_key_pack {
	struct encryption_key reply_key;
	SECItem nonce;
};

struct checksum {
	SECItem cksumtype;
	SECItem checksum;
};

struct reply_key_pack {
	struct encryption_key reply_key;
	struct checksum as_checksum;
};

struct kdc_req {
	SECItem pvno;
	SECItem msg_type;
	SECItem **padata;
	SECItem req_body;
};

struct authorization_datum {
	SECItem ad_type;
	SECItem ad_data;
};
struct typed_datum {
	SECItem td_type;
	SECItem td_data;
};

extern const SEC_ASN1Template pkinit_kerberos_principal_name_template[];

SECItem *pkinit_encode_auth_pack(struct module_context *mcontext,
				 PLArenaPool *pool,
				 struct auth_pack *auth_pack);
SECItem *pkinit_encode_draft_auth_pack(struct module_context *mcontext,
				       PLArenaPool *pool,
				       struct draft_auth_pack *auth_pack);
SECItem *pkinit_encode_draft_pa_pk_as_rep(struct module_context *mcontext,
					  PLArenaPool *pool,
					  struct draft_pa_pk_as_rep *rep);
SECItem *pkinit_encode_draft_pa_pk_as_req(struct module_context *mcontext,
					  PLArenaPool *pool,
					  struct draft_pa_pk_as_req *req);
SECItem *pkinit_encode_draft_reply_key_pack(struct module_context *mcontext,
					    PLArenaPool *pool,
					    struct draft_reply_key_pack *kpack);
SECItem *pkinit_encode_kerberos_principal_name(struct module_context *mcontext,
					       PLArenaPool *pool,
					       struct kerberos_principal_name *name);
SECItem *pkinit_encode_pa_pk_as_rep(struct module_context *mcontext,
				    PLArenaPool *pool,
				    struct pa_pk_as_rep *rep);
SECItem *pkinit_encode_pa_pk_as_req(struct module_context *mcontext,
				    PLArenaPool *pool,
				    struct pa_pk_as_req *req);
SECItem *pkinit_encode_reply_key_pack(struct module_context *mcontext,
				      PLArenaPool *pool,
				      struct reply_key_pack *key_pack);
SECItem *pkinit_encode_kdc_dh_key_info(struct module_context *mcontext,
				       PLArenaPool *pool,
				       struct kdc_dh_key_info *key_info);
SECItem *pkinit_encode_authorization_data(struct module_context *mcontext,
				          PLArenaPool *pool,
					  struct authorization_datum **adata);
SECItem *pkinit_encode_authorization_datum(struct module_context *mcontext,
					   PLArenaPool *pool,
					   struct authorization_datum *adata);
SECItem *pkinit_encode_typed_data(struct module_context *mcontext,
			          PLArenaPool *pool,
				  struct typed_datum **adata);
SECItem *pkinit_encode_typed_datum(struct module_context *mcontext,
				   PLArenaPool *pool,
				   struct typed_datum *adata);
struct auth_pack *pkinit_decode_auth_pack(struct module_context *mcontext,
					  PLArenaPool *pool, SECItem *data);
struct draft_auth_pack *pkinit_decode_draft_auth_pack(struct module_context *mcontext,
						      PLArenaPool *pool,
						      SECItem *data);
struct draft_pa_pk_as_rep *pkinit_decode_draft_pa_pk_as_rep(struct module_context *mcontext,
							    PLArenaPool *pool,
							    SECItem *data);
struct draft_pa_pk_as_req *pkinit_decode_draft_pa_pk_as_req(struct module_context *mcontext,
							    PLArenaPool *pool,
							    SECItem *data);
struct draft_reply_key_pack *pkinit_decode_draft_reply_key_pack(struct module_context *mcontext,
								PLArenaPool *pool,
								SECItem *data);
struct kerberos_principal_name *pkinit_decode_kerberos_principal_name(struct module_context *mcontext,
								      PLArenaPool *pool,
								      SECItem *data);
struct pa_pk_as_rep *pkinit_decode_pa_pk_as_rep(struct module_context *mcontext,
						PLArenaPool *pool,
						SECItem *data);
struct pa_pk_as_req *pkinit_decode_pa_pk_as_req(struct module_context *mcontext,
						PLArenaPool *pool,
						SECItem *data);
struct reply_key_pack *pkinit_decode_reply_key_pack(struct module_context *mcontext,
						    PLArenaPool *pool,
						    SECItem *data);
struct kdc_dh_key_info *pkinit_decode_kdc_dh_key_info(struct module_context *mcontext,
						      PLArenaPool *pool,
						      SECItem *item);
struct authorization_datum **pkinit_decode_authorization_data(struct module_context *mcontext,
							      PLArenaPool *pool,
							      SECItem *item);
struct authorization_datum *pkinit_decode_authorization_datum(struct module_context *mcontext,
							      PLArenaPool *pool,
							      SECItem *item);
struct typed_datum **pkinit_decode_typed_data(struct module_context *mcontext,
					      PLArenaPool *pool,
					      SECItem *item);
struct typed_datum *pkinit_decode_typed_datum(struct module_context *mcontext,
					      PLArenaPool *pool,
					      SECItem *item);

krb5_error_code pkinit_build_tgs_name(struct module_context *mcontext,
				      krb5_context kcontext,
				      krb5_principal member,
				      krb5_principal *tgs_principal_name);

krb5_data *pkinit_create_draft_pa_pk_as_req(struct module_context *mcontext,
					    PLArenaPool *pool,
					    krb5_context kcontext,
					    krb5_kdc_req *request,
					    CERTCertDBHandle *certdb,
					    struct aa_bag *bag,
					    CERTCertificate *cert,
					    int signed_data_version,
					    PK11SlotInfo *preferred_slot,
					    int preferred_oakley_group,
					    SECKEYPrivateKey **dh_private_key,
					    SECKEYPublicKey **dh_public_key,
					    SECItem **dh_client_nonce,
					    PK11PasswordFunc pwcb,
					    struct pkinit_pwcb_args *pwcb_args);
krb5_data *pkinit_create_pa_pk_as_req(struct module_context *mcontext,
				      PLArenaPool *pool,
				      krb5_context kcontext,
				      krb5_kdc_req *request,
				      krb5_data *encoded_request_body,
				      CERTCertDBHandle *certdb,
				      struct aa_bag *bag,
				      CERTCertificate *cert,
				      int signed_data_version,
				      SECOidTag digest,
				      SECOidTag signature,
				      PK11SlotInfo *preferred_slot,
				      int preferred_oakley_group,
				      struct domain_parameters **allowed_dh_params,
				      unsigned int minimum_dh_prime_size,
				      SECKEYPrivateKey **dh_private_key,
				      SECKEYPublicKey **dh_public_key,
				      SECItem **dh_client_nonce,
				      PK11PasswordFunc pwcb,
				      struct pkinit_pwcb_args *pwcb_args);
krb5_error_code pkinit_verify_pa_pk_as_req(struct module_context *mcontext,
					   PLArenaPool *pool,
					   CERTCertDBHandle *certdb,
					   struct aa_bag *bag,
					   krb5_context kcontext,
					   krb5_kdc_req *request,
					   krb5_data *request_pkt,
					   krb5_data *padata,
					   krb5_deltat max_time_skew,
					   unsigned int minimum_dh_prime_length,
					   SECItem **authorized_certs,
					   int n_authorized_certs,
					   const char **additional_authzdn_list,
					   PRInt32 client_cert_flags,
					   CERTCertificate **signing_cert,
					   unsigned long *signed_data_version,
					   struct subject_public_key_info **client_dh_key,
					   SECItem **nonce,
					   SECItem **client_dh_nonce,
					   krb5_authdata **adata,
					   krb5_data **e_data);
krb5_data *pkinit_create_pa_pk_as_rep(struct module_context *mcontext,
				      PLArenaPool *pool,
				      CERTCertDBHandle *certdb,
				      struct aa_bag *bag,
				      CERTCertificate *kdc_certificate,
				      int signed_data_version,
				      krb5_context kcontext,
				      krb5_keyblock *encrypting_key,
				      krb5_data *encoded_request,
				      SECItem *client_cert,
				      SECOidTag digest,
				      SECOidTag signature,
				      struct subject_public_key_info *client_public_key,
				      SECItem *nonce,
				      SECItem *client_dh_nonce);
krb5_error_code pkinit_verify_draft_pa_pk_as_rep(struct module_context *mcontext,
						 PLArenaPool *pool,
						 CERTCertDBHandle *certdb,
						 struct aa_bag *bag,
						 PK11SlotInfo *preferred_slot,
						 PK11PasswordFunc pwcb,
						 struct pkinit_pwcb_args *pwcb_args,
						 const char **additional_authzdn_list,
						 PRInt32 kdc_cert_flags,
						 krb5_context kcontext,
						 krb5_kdc_req *request,
						 krb5_data *padata,
						 krb5_keyblock *encrypting_key);
krb5_error_code pkinit_verify_pa_pk_as_rep(struct module_context *mcontext,
					   PLArenaPool *pool,
					   CERTCertDBHandle *certdb,
					   struct aa_bag *bag,
					   PK11SlotInfo *preferred_slot,
					   PK11PasswordFunc pwcb,
					   struct pkinit_pwcb_args *pwcb_args,
					   const char **additional_authzdn_list,
					   PRInt32 kdc_cert_flags,
					   krb5_context kcontext,
					   krb5_kdc_req *request,
					   krb5_data *request_pkt,
					   krb5_data *padata,
					   SECKEYPrivateKey *dh_private_key,
					   SECKEYPublicKey *dh_public_key,
					   SECItem *dh_client_nonce,
					   krb5_keyblock *encrypting_key);

SECStatus pkinit_generalized_time_from_time(struct module_context *mcontext,
					    PLArenaPool *pool,
					    time_t *when, SECItem *ktime);
time_t pkinit_time_from_generalized_time(struct module_context *mcontext,
					 SECItem *item);

SECStatus pkinit_utc_time_from_time(struct module_context *mcontext,
				    PLArenaPool *pool,
				    time_t *when, SECItem *ktime);
time_t pkinit_time_from_utc_time(struct module_context *mcontext,
				 SECItem *item);

SECStatus pkinit_signing_time_from_time(struct module_context *mcontext,
					PLArenaPool *pool,
					time_t *when, SECItem *ktime);
time_t pkinit_time_from_signing_time(struct module_context *mcontext,
				     PLArenaPool *pool, SECItem *item);

#endif
